@@ -759,6 +759,8 @@ class database:
 		curs.execute("alter database %s set check_function_bodies to on" % self.name)
 		# we want checking of data checksums if available
 		curs.execute("alter database %s set ignore_checksum_failure to off" % self.name)
+		# tighten permissions on schema public
+		curs.execute("revoke create on schema public from public")
 		curs.close()
 		self.conn.commit()
 
@@ -780,9 +782,19 @@ class database:
 			curs.execute("alter database %s set track_commit_timestamp to on" % self.name)
 		except:
 			_log.exception(u'PostgreSQL version < 9.5 does not support <track_commit_timestamp> OR <track_commit_timestamp> cannot be set at runtime')
-
 		curs.close()
 		self.conn.commit()
+
+		# set owner of schema public to new role "pg_database_owner",
+		# as suggested by PG 15 release notes
+		curs = self.conn.cursor()
+		try:
+			curs.execute("alter schema public owner to pg_database_owner")
+		except:
+			_log.exception(u'PostgreSQL versions < 15 do not yet support role <pg_database_owner>')
+		curs.close()
+		self.conn.commit()
+
 		curs = self.conn.cursor()
 		gmConnectionPool.log_pg_settings(curs = curs)
 		curs.close()
@@ -833,7 +845,7 @@ class database:
 
 	#--------------------------------------------------------------
 	def __create_db(self):
-
+		_log.info('creating database')
 		# verify template database hash
 		template_version = cfg_get(self.section, 'template version')
 		if template_version is None:
@@ -843,6 +855,7 @@ class database:
 			if not converted:
 				_log.error(u'invalid template database definition: %s', template_version)
 				return False
+
 			if not gmPG2.database_schema_compatible(link_obj = self.conn, version = version):
 				_log.error(u'invalid [%s] schema structure in GNUmed template database [%s]', template_version, self.template_db)
 				return False
